"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../runtime/dsl2");
require("../watchers/system");
var prog = new dsl2_1.Program("clock");
prog.attach("system");
prog.attach("svg");
// ~~~
// search @browser
//   hand = [#clock-hand degrees length]
//   x2 = 50 + (length * sin[degrees])
//   y2 = 50 - (length * cos[degrees])
// bind @browser
//   hand <- [#line, x1: 50, y1: 50, x2, y2]
// ~~~
// Draw a clock using SVG. We find the angle for each hand using time and let the `#clock-hand` block take care of the rest.
// ~~~
// search
//   [#time hours minutes seconds]
// bind @browser
//   [#svg viewBox: "0 0 100 100", width: "300px", children:
//     [#circle cx: 50, cy: 50, r: 45, fill: "#0B79CE"]
//     [#clock-hand #hour-hand degrees: 30 * hours, length: 30, stroke: "#023963"]
//     [#clock-hand #minute-hand degrees: 6 * minutes, length: 40, stroke: "#023963"]
//     [#clock-hand #second-hand degrees: 6 * seconds, length: 40, stroke: "#ce0b46"]]
// ~~~
prog.block("draw a clock", function (_a) {
    var find = _a.find, record = _a.record;
    var timer = find("clock-timer");
    return [
        record("svg", { viewBox: "0 0 100 100", width: "300px", timer: timer }).add("children", [
            record("svg/circle", { cx: 50, cy: 50, r: 45, fill: "#0b79ce" }),
            record("clock-hand", "hour-hand", { timer: timer, length: 30, stroke: "black" }).add("degrees", 30 * timer.hours),
            record("clock-hand", "minute-hand", { timer: timer, length: 40, stroke: "black" }).add("degrees", 6 * timer.minutes),
            record("clock-hand", "second-hand", { timer: timer, length: 40, stroke: "red" }).add("degrees", 6 * timer.seconds),
        ])
    ];
});
prog.block("draw clock hands", function (_a) {
    var find = _a.find, lib = _a.lib;
    var math = lib.math;
    var hand = find("clock-hand");
    var x2 = 50 + hand.length * math.sin(hand.degrees);
    var y2 = 50 - hand.length * math.cos(hand.degrees);
    return [
        hand.add("tag", "svg/line")
            .add("x1", 50)
            .add("y1", 50)
            .add("x2", x2)
            .add("y2", y2)
    ];
});
prog
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg");
    return [elem.add("tag", "svg/element").add("tagname", "svg")];
})
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg/circle");
    return [elem.add("tag", "svg/element").add("tagname", "circle")];
})
    .block("Translate elements into svg", function (_a) {
    var find = _a.find, record = _a.record, union = _a.union;
    var elem = find("svg/line");
    return [elem.add("tag", "svg/element").add("tagname", "line")];
});
prog.test(1, [
    [1, "tag", "clock-timer"],
    [1, "tag", "system/timer"],
    [1, "resolution", 1000],
    [2, "tag", "clock-timer"],
    [2, "tag", "system/timer"],
    [2, "resolution", 5000],
]);
// prog.test(2, [
//   [1, "seconds", 10, 0, -1],
//   [1, "seconds", 11],
// ])
// prog.test(3, [
//   [1, "seconds", 11, 0, -1],
//   [1, "seconds", 12],
// ])
//# sourceMappingURL=clock.js.map