//--------------------------------------------------------------------
// Flappy
//--------------------------------------------------------------------
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var dsl2_1 = require("../runtime/dsl2");
require("../watchers/system");
var node_uuid_1 = require("node-uuid");
//--------------------------------------------------------------------
// Utils
//--------------------------------------------------------------------
function toEAVs(eavs, obj) {
    var record = node_uuid_1.v4();
    for (var attribute in obj) {
        var values = obj[attribute];
        if (values.constructor === Array) {
            for (var _i = 0, values_1 = values; _i < values_1.length; _i++) {
                var value = values_1[_i];
                eavs.push([record, attribute, value]);
            }
        }
        else {
            eavs.push([record, attribute, values]);
        }
    }
    return eavs;
}
//--------------------------------------------------------------------
// Program
//--------------------------------------------------------------------
var prog = new dsl2_1.Program("flappy");
prog.attach("system");
//--------------------------------------------------------------------
// Start the game
//--------------------------------------------------------------------
prog.commit("clicking starts the game", function (_a) {
    var find = _a.find, record = _a.record, lib = _a.lib, choose = _a.choose;
    var math = lib.math;
    var world = find("world");
    // let svg = find("game-window");
    // find("html/event/click", {element:svg});
    find("html/event/click");
    choose(function () { world.screen == "menu"; }, function () { world.screen == "game over"; });
    var bestScore = world.best;
    var player = find("player");
    return [
        world.remove("screen").add("screen", "game"),
        //      .remove("distance").add("distance", 0)
        //      .remove("best").add("best", bestScore),
        player //.remove("x").add("x", 25)
            .remove("velocity").add("velocity", 0)
    ];
});
//--------------------------------------------------------------------
// Flapping the player
//--------------------------------------------------------------------
prog.commit("apply a velocity when you click", function (_a) {
    var find = _a.find;
    var world = find("world", { screen: "game" });
    find("html/event/click");
    var player = find("player", "self");
    return [
        player.remove("velocity").add("velocity", 1.17)
    ];
});
//--------------------------------------------------------------------
// Scroll the world
//--------------------------------------------------------------------
prog.commit("scroll the world", function (_a) {
    var find = _a.find, not = _a.not;
    var frame = find("frames").frame;
    var world = find("world", { screen: "game" });
    frame != world.frame;
    var player = find("player");
    world.distance;
    var adjust = 1 / 60;
    not(function () { find("html/event/click"); });
    return [
        world.remove("frame").add("frame", frame)
            .remove("distance").add("distance", world.distance + adjust),
        player.remove("y").add("y", player.y - player.velocity)
            .remove("velocity").add("velocity", player.velocity + world.gravity)
    ];
});
//--------------------------------------------------------------------
// svg/html translation
//--------------------------------------------------------------------
prog.commit("Remove click events!", function (_a) {
    var find = _a.find;
    var click = find("html/event/click");
    return [
        //click.remove("tag")
        click.remove("tag"),
    ];
});
//--------------------------------------------------------------------
// Go!
//--------------------------------------------------------------------
var changes = [];
// toEAVs(changes, {tag:["frames", "system/timer"], resolution:1000})
toEAVs(changes, { tag: ["player", "self"], name: "eve", x: 25, y: 50, velocity: 0 });
toEAVs(changes, { tag: "world", screen: "menu", frame: 0, distance: 0, best: 0, gravity: -0.061 });
console.groupCollapsed("start");
prog.inputEavs(changes);
console.groupEnd();
console.groupCollapsed("add frames");
prog.inputEavs([["meep", "tag", "frames"], ["meep", "frame", 1]]);
console.groupEnd();
console.groupCollapsed("click");
prog.inputEavs(toEAVs([], { tag: "html/event/click" }));
console.groupEnd();
console.groupCollapsed("frame tick 2");
prog.inputEavs([["meep", "frame", 2], ["meep", "frame", 1, -1]]);
console.groupEnd();
// prog.inputEavs([["meep", "frame", 3], ["meep", "frame", 2, -1]])
// prog.inputEavs(toEAVs([], {tag: "html/event/click"}));
// prog.inputEavs([["meep", "frame", 4], ["meep", "frame", 3, -1]])
// prog.inputEavs([["meep", "frame", 5], ["meep", "frame", 4, -1]])
// console.log(prog);
//# sourceMappingURL=test.js.map