//--------------------------------------------------------------------
// The Eve compiler as a watcher
//--------------------------------------------------------------------
"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var dsl2_1 = require("../runtime/dsl2");
require("setimmediate");
var CompilerWatcher = (function (_super) {
    __extends(CompilerWatcher, _super);
    function CompilerWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.blocksToCompile = {};
        _this.blocksToRemove = {};
        _this.blocks = {};
        _this.items = {};
        _this.watcherFunctions = {};
        _this.programToInjectInto = _this.program;
        //------------------------------------------------------------------
        // Compile queue
        //------------------------------------------------------------------
        _this.queued = false;
        _this.runQueue = function () {
            var adds = [];
            var removes = [];
            for (var ID in _this.blocksToRemove) {
                if (!_this.blocks[ID])
                    continue;
                removes.push(_this.blocks[ID]);
                delete _this.blocks[ID];
            }
            for (var ID in _this.blocksToCompile) {
                if (!_this.items[ID])
                    continue;
                var neue = _this.compileBlock(ID);
                adds.push(neue);
                _this.blocks[ID] = neue;
            }
            _this.programToInjectInto.blockChangeTransaction(adds, removes);
            _this.queued = false;
            _this.blocksToCompile = {};
            _this.blocksToRemove = {};
        };
        _this.compileValue = function (compile, context, value) {
            if (value === undefined)
                return undefined;
            var items = _this.items;
            if (items[value] && items[value].type === "variable") {
                var found = compile.variables[value];
                if (!found) {
                    found = compile.variables[value] = new dsl2_1.Reference(context);
                }
                return found;
            }
            return value;
        };
        return _this;
    }
    CompilerWatcher.prototype.queue = function (blockID, isAdd) {
        if (isAdd === void 0) { isAdd = true; }
        if (isAdd)
            this.blocksToCompile[blockID] = true;
        this.blocksToRemove[blockID] = true;
        if (!this.queued) {
            this.queued = true;
            setImmediate(this.runQueue);
        }
    };
    //------------------------------------------------------------------
    // Program to inject into
    //------------------------------------------------------------------
    CompilerWatcher.prototype.injectInto = function (prog) {
        this.programToInjectInto = prog;
    };
    //------------------------------------------------------------------
    // Watch functions
    //------------------------------------------------------------------
    CompilerWatcher.prototype.registerWatcherFunction = function (name, consumer) {
        this.watcherFunctions[name] = consumer;
    };
    //------------------------------------------------------------------
    // Compiler
    //------------------------------------------------------------------
    CompilerWatcher.prototype.inContext = function (flow, func) {
        dsl2_1.ReferenceContext.push(flow.context);
        func();
        dsl2_1.ReferenceContext.pop();
    };
    CompilerWatcher.prototype.compileBlock = function (blockID) {
        var _a = this, inContext = _a.inContext, items = _a.items, compileValue = _a.compileValue;
        var item = items[blockID];
        var name = item.name, constraints = item.constraints, type = item.type;
        var compile = { variables: {} };
        var flow;
        if (type === "commit") {
            flow = new dsl2_1.CommitFlow(function (a) { return []; });
        }
        else if (type === "watch") {
            flow = new dsl2_1.WatchFlow(function (a) { return []; });
        }
        else {
            flow = new dsl2_1.LinearFlow(function (a) { return []; });
        }
        var context = flow.context;
        var _loop_1 = function (constraintID) {
            var constraint = items[constraintID];
            if (!constraint)
                return "continue";
            if (constraint.type === "record") {
                inContext(flow, function () {
                    var attrs = {};
                    for (var _i = 0, _a = constraint.attributes; _i < _a.length; _i++) {
                        var _b = _a[_i], attribute = _b[0], value = _b[1];
                        var safeValue = compileValue(compile, context, value);
                        var found = attrs[attribute];
                        if (!found) {
                            found = attrs[attribute] = [];
                        }
                        found.push(safeValue);
                    }
                    var recordVar = compileValue(compile, context, constraint.record);
                    var record = new dsl2_1.Record(flow.context, [], attrs, recordVar);
                    recordVar.__owner = record;
                });
            }
            if (constraint.type === "output") {
                inContext(flow, function () {
                    var attrs = {};
                    for (var _i = 0, _a = constraint.attributes; _i < _a.length; _i++) {
                        var _b = _a[_i], attribute = _b[0], value = _b[1];
                        var safeValue = compileValue(compile, context, value);
                        var found = attrs[attribute];
                        if (!found) {
                            found = attrs[attribute] = [];
                        }
                        found.push(safeValue);
                    }
                    var outputType = dsl2_1.Insert;
                    var outputOp = "add";
                    if (constraint.outputType === "remove") {
                        outputType = dsl2_1.Remove;
                        outputOp = "remove";
                    }
                    var outputVar = compileValue(compile, context, constraint.record);
                    var output;
                    if (!outputVar.__owner) {
                        output = new outputType(flow.context, [], attrs);
                        context.equality(output.reference(), outputVar);
                    }
                    else {
                        output = new outputType(flow.context, [], attrs, outputVar);
                    }
                    var record = output.reference();
                    for (var _c = 0, _d = constraint.nonIdentityAttribute; _c < _d.length; _c++) {
                        var _e = _d[_c], attribute = _e[0], value = _e[1];
                        record[outputOp](attribute, compileValue(compile, context, value));
                    }
                });
            }
            if (constraint.type === "removeRecord") {
                inContext(flow, function () {
                    var outputVar = compileValue(compile, context, constraint.record);
                    var output;
                    if (!outputVar.__owner) {
                        throw new Error("Trying to fully remove a record that doesn't exist");
                    }
                    outputVar.remove();
                });
            }
            if (constraint.type === "lookup") {
                inContext(flow, function () {
                    var lookup = flow.lookup(compileValue(compile, context, constraint.record));
                    context.equality(lookup.attribute, compileValue(compile, context, constraint.attribute));
                    context.equality(lookup.value, compileValue(compile, context, constraint.value));
                    console.log("LOOKUP", lookup);
                });
            }
        };
        for (var _i = 0, constraints_1 = constraints; _i < constraints_1.length; _i++) {
            var constraintID = constraints_1[_i];
            _loop_1(constraintID);
        }
        var block = this.programToInjectInto["_" + type](name, flow);
        if (type === "watch" && item.watcher) {
            var func = this.watcherFunctions[item.watcher];
            if (!func) {
                console.error("No such watcher function registered: " + item.watcher);
            }
            else {
                this.programToInjectInto.asDiffs(func);
            }
        }
        console.log("Compiled: ", block);
        return block;
    };
    //------------------------------------------------------------------
    // Compile item extraction via watch blocks
    //------------------------------------------------------------------
    CompilerWatcher.prototype.setup = function () {
        var _this = this;
        var me = this.program;
        me.watch("get blocks", function (_a) {
            var find = _a.find, record = _a.record;
            var block = find("eve/compiler/block");
            var constraint = block.constraint, name = block.name, type = block.type;
            return [
                record({ block: block, constraint: constraint, name: name, type: type })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, name_1 = _b.name, constraint = _b.constraint, type = _b.type;
                var found = items[block];
                if (!found) {
                    found = items[block] = { type: type, name: name_1, constraints: [] };
                }
                found.name = name_1;
                if (found.constraints.indexOf(constraint) === -1) {
                    found.constraints.push(constraint);
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, name_2 = _c.name, constraint = _c.constraint;
                var found = items[block];
                if (!found) {
                    continue;
                }
                var ix = found.constraints.indexOf(constraint);
                if (ix > -1) {
                    found.constraints.splice(ix, 1);
                }
                if (found.constraints.length === 0) {
                    delete items[block];
                }
                _this.queue(block);
            }
        });
        me.watch("get watcher property", function (_a) {
            var find = _a.find, record = _a.record;
            var block = find("eve/compiler/block");
            var constraint = block.constraint, name = block.name, type = block.type, watcher = block.watcher;
            return [
                record({ block: block, watcher: watcher })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, watcher = _b.watcher;
                var found = items[block];
                found.watcher = watcher;
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, watcher = _c.watcher;
                var found = items[block];
                if (!found) {
                    continue;
                }
                found.watcher = undefined;
                _this.queue(block);
            }
        });
        me.watch("get variables", function (_a) {
            var find = _a.find, record = _a.record;
            var variable = find("eve/compiler/var");
            return [
                record({ variable: variable })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var variable = adds[key].variable;
                items[variable] = { type: "variable" };
            }
        });
        me.watch("get lookups", function (_a) {
            var find = _a.find, record = _a.record;
            var lookup = find("eve/compiler/lookup");
            var block = find("eve/compiler/block", { constraint: lookup });
            var rec = lookup.record, attribute = lookup.attribute, value = lookup.value;
            return [
                record({ block: block, id: lookup, record: rec, attribute: attribute, value: value })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute, value = _b.value;
                items[id] = { type: "lookup", record: record, attribute: attribute, value: value };
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id, record = _c.record, attribute = _c.attribute, value = _c.value;
                delete items[id];
                _this.queue(block);
            }
        });
        me.watch("get records", function (_a) {
            var find = _a.find, record = _a.record;
            var compilerRecord = find("eve/compiler/record");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            var id = compilerRecord.record, attribute = compilerRecord.attribute;
            return [
                record({ block: block, id: compilerRecord, record: id, attribute: attribute.attribute, value: attribute.value })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute, value = _b.value;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "record", attributes: [], record: record };
                }
                found.attributes.push([attribute, value]);
                _this.queue(block);
            }
            var _loop_2 = function (key) {
                var _a = removes[key], block = _a.block, id = _a.id, record = _a.record, attribute = _a.attribute, value = _a.value;
                var found = items[id];
                if (!found) {
                    return "continue";
                }
                found.attributes = found.attributes.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== value;
                });
                if (found.attributes.length === 0) {
                    delete items[id];
                }
                _this.queue(block);
            };
            for (var key in removes) {
                _loop_2(key);
            }
        });
        me.watch("get outputs", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose;
            var compilerRecord = find("eve/compiler/output");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            var id = compilerRecord.record, attribute = compilerRecord.attribute;
            var attributeType = choose(function () {
                attribute.tag == "eve/compiler/attribute/non-identity";
                return "non-identity";
            }, function () {
                return "identity";
            })[0];
            var outputType = choose(function () {
                compilerRecord.tag == "eve/compiler/remove";
                return "remove";
            }, function () {
                return "add";
            })[0];
            return [
                record({ block: block, id: compilerRecord, record: id, attribute: attribute.attribute, value: attribute.value, attributeType: attributeType, outputType: outputType })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record, attribute = _b.attribute, value = _b.value, attributeType = _b.attributeType, outputType = _b.outputType;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "output", attributes: [], nonIdentityAttribute: [], record: record, outputType: outputType };
                }
                if (attributeType === "identity") {
                    found.attributes.push([attribute, value]);
                }
                else {
                    found.nonIdentityAttribute.push([attribute, value]);
                }
                _this.queue(block);
            }
            var _loop_3 = function (key) {
                var _a = removes[key], block = _a.block, id = _a.id, record = _a.record, attribute = _a.attribute, value = _a.value;
                var found = items[id];
                if (!found) {
                    return "continue";
                }
                found.attributes = found.attributes.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== value;
                });
                found.nonIdentityAttribute = found.nonIdentityAttribute.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== value;
                });
                if (found.attributes.length === 0) {
                    delete items[id];
                }
                _this.queue(block);
            };
            for (var key in removes) {
                _loop_3(key);
            }
        });
        me.watch("get valueless outputs", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose, not = _a.not;
            var compilerRecord = find("eve/compiler/output");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            var attribute = compilerRecord.attribute;
            not(function () { return attribute.value; });
            var attributeType = choose(function () {
                attribute.tag == "eve/compiler/attribute/non-identity";
                return "non-identity";
            }, function () {
                return "identity";
            })[0];
            return [
                record({ block: block, id: compilerRecord, attribute: attribute.attribute, attributeType: attributeType })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, attribute = _b.attribute, attributeType = _b.attributeType;
                var found = items[id];
                if (!found) {
                    console.warn("This seems pretty bad.");
                    continue;
                }
                if (attributeType === "identity") {
                    found.attributes.push([attribute, undefined]);
                }
                else {
                    found.nonIdentityAttribute.push([attribute, undefined]);
                }
                _this.queue(block);
            }
            var _loop_4 = function (key) {
                var _a = removes[key], block = _a.block, id = _a.id, attribute = _a.attribute;
                var found = items[id];
                if (!found) {
                    return "continue";
                }
                found.attributes = found.attributes.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== undefined;
                });
                found.nonIdentityAttribute = found.nonIdentityAttribute.filter(function (_a) {
                    var a = _a[0], v = _a[1];
                    return a !== attribute || v !== undefined;
                });
                if (found.attributes.length === 0) {
                    delete items[id];
                }
                _this.queue(block);
            };
            for (var key in removes) {
                _loop_4(key);
            }
        });
        me.watch("get full remove", function (_a) {
            var find = _a.find, record = _a.record, choose = _a.choose, not = _a.not;
            var compilerRecord = find("eve/compiler/remove");
            var block = find("eve/compiler/block", { constraint: compilerRecord });
            not(function () { return compilerRecord.attribute; });
            return [
                record({ block: block, id: compilerRecord, record: compilerRecord.record })
            ];
        });
        me.asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            var items = _this.items;
            for (var key in adds) {
                var _b = adds[key], block = _b.block, id = _b.id, record = _b.record;
                var found = items[id];
                if (!found) {
                    found = items[id] = { type: "removeRecord", record: record };
                }
                _this.queue(block);
            }
            for (var key in removes) {
                var _c = removes[key], block = _c.block, id = _c.id;
                var found = items[id];
                if (!found) {
                    continue;
                }
                delete items[id];
                _this.queue(block);
            }
        });
    };
    return CompilerWatcher;
}(watcher_1.Watcher));
exports.CompilerWatcher = CompilerWatcher;
watcher_1.Watcher.register("compiler", CompilerWatcher);
//# sourceMappingURL=compiler.js.map