"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var watcher_1 = require("./watcher");
var dom_1 = require("./dom");
var node_uuid_1 = require("node-uuid");
var HTMLWatcher = (function (_super) {
    __extends(HTMLWatcher, _super);
    function HTMLWatcher() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.tagPrefix = "html";
        _this.sentInputValues = {};
        return _this;
    }
    HTMLWatcher.prototype.createInstance = function (id, element, tagname) {
        var elem = document.createElement(tagname);
        elem.setAttribute("instance", "" + watcher_1.maybeIntern(id));
        elem.setAttribute("element", "" + watcher_1.maybeIntern(element));
        elem.__element = element;
        elem.__styles = [];
        return elem;
    };
    HTMLWatcher.prototype.createRoot = function (id) {
        var elem = this.instances[id];
        if (!elem)
            throw new Error("Orphaned instance '" + id + "'");
        document.body.appendChild(elem);
        return elem;
    };
    HTMLWatcher.prototype.addAttribute = function (instance, attribute, value) {
        // @TODO: Error checking to ensure we don't double-set attributes.
        if (attribute == "value") {
            if (instance.classList.contains("html-autosize-input") && instance instanceof HTMLInputElement) {
                instance.size = (instance.value || "").length || 1;
            }
            instance.value = "" + watcher_1.maybeIntern(value);
        }
        else if (attribute == "tag") {
            if (value === "html/autosize-input" && instance instanceof HTMLInputElement) {
                setImmediate(function () { return instance.size = (instance.value || "").length || 1; });
            }
            else if (value === "html/trigger-focus" && instance instanceof HTMLInputElement) {
                setImmediate(function () { return instance.focus(); });
            }
            else {
                instance.setAttribute(attribute, "" + watcher_1.maybeIntern(value));
            }
        }
        else {
            instance.setAttribute(attribute, "" + watcher_1.maybeIntern(value));
        }
    };
    HTMLWatcher.prototype.removeAttribute = function (instance, attribute, value) {
        // @TODO: Error checking to ensure we don't double-remove attributes or remove the wrong value.
        instance.removeAttribute(attribute);
        if (attribute === "value") {
            var input = instance;
            if (input.value === value)
                input.value = "";
        }
    };
    HTMLWatcher.prototype.setup = function () {
        var _this = this;
        if (typeof window === "undefined")
            return;
        this.tagPrefix = "html"; // @FIXME: hacky, due to inheritance chain evaluation order.
        _super.prototype.setup.call(this);
        this.program
            .bind("All html elements add their tags as classes", function (_a) {
            var find = _a.find, string = _a.lib.string, record = _a.record;
            var element = find("html/element");
            element.tag != "html/element";
            var klass = string.replace(element.tag, "/", "-");
            return [
                element.add("class", klass)
            ];
        });
        window.addEventListener("click", function (event) {
            var target = event.target;
            if (!_this.isInstance(target))
                return;
            var eavs = [];
            var current = target;
            while (current && _this.isInstance(current)) {
                var elemId = current.__element;
                var eventId = node_uuid_1.v4();
                eavs.push([eventId, "tag", "html/event/click"], [eventId, "element", elemId]);
                if (current === target) {
                    eavs.push([eventId, "tag", "html/direct-target"]);
                }
                current = current.parentElement;
            }
            _this._sendEvent(eavs);
        });
        window.addEventListener("input", function (event) {
            var target = event.target;
            var elementId = target.__element;
            if (elementId) {
                if (target.classList.contains("html-autosize-input")) {
                    target.size = target.value.length || 1;
                }
                var sentInputValues = _this.sentInputValues;
                if (!sentInputValues[elementId]) {
                    sentInputValues[elementId] = [];
                }
                sentInputValues[elementId].push(target.value);
                var eventId = node_uuid_1.v4();
                var eavs = [
                    [eventId, "tag", "html/event/change"],
                    [eventId, "element", elementId],
                    [eventId, "value", target.value]
                ];
                _this._sendEvent(eavs);
            }
        });
        window.addEventListener("focus", function (event) {
            var target = event.target;
            var elementId = target.__element;
            if (elementId) {
                var eventId = node_uuid_1.v4();
                var eavs = [
                    [eventId, "tag", "html/event/focus"],
                    [eventId, "element", elementId]
                ];
                if (target.value !== undefined)
                    eavs.push([eventId, "value", target.value]);
                _this._sendEvent(eavs);
            }
        }, true);
        window.addEventListener("blur", function (event) {
            var target = event.target;
            var elementId = target.__element;
            if (elementId) {
                var eventId = node_uuid_1.v4();
                var eavs = [
                    [eventId, "tag", "html/event/blur"],
                    [eventId, "element", elementId]
                ];
                if (target.value !== undefined)
                    eavs.push([eventId, "value", target.value]);
                _this._sendEvent(eavs);
            }
        }, true);
        document.body.addEventListener("mouseenter", function (event) {
            var target = event.target;
            if (!_this.isInstance(target))
                return;
            var eavs = [];
            var elemId = target.__element;
            if (target.listeners && target.listeners["hover"]) {
                var eventId = node_uuid_1.v4();
                eavs.push([eventId, "tag", "html/event/hover-in"], [eventId, "element", elemId]);
            }
            if (eavs.length)
                _this._sendEvent(eavs);
        }, true);
        document.body.addEventListener("mouseleave", function (event) {
            var target = event.target;
            if (!_this.isInstance(target))
                return;
            var eavs = [];
            var elemId = target.__element;
            if (target.listeners && target.listeners["hover"]) {
                var eventId = node_uuid_1.v4();
                eavs.push([eventId, "tag", "html/event/hover-out"], [eventId, "element", elemId]);
            }
            if (eavs.length)
                _this._sendEvent(eavs);
        }, true);
        this.program
            .commit("Remove input-related events.", function (_a) {
            var find = _a.find, choose = _a.choose;
            var event = choose(function () { return find("html/event/change"); }, function () { return find("html/event/focus"); }, function () { return find("html/event/blur"); })[0];
            event.tag;
            return [event.remove()];
        })
            .bind("Inputs with an initial but no value use the initial.", function (_a) {
            var find = _a.find, choose = _a.choose;
            var input = find("html/element", { tagname: "input" });
            var value = choose(function () { return input.value; }, function () { return input.initial; })[0];
            return [input.add("value", value)];
        })
            .commit("Apply input value changes.", function (_a) {
            var find = _a.find;
            var _b = find("html/event/change"), element = _b.element, value = _b.value;
            return [element.remove("value").add("value", value)];
        });
        this.program
            .commit("When an element is entered, mark it hovered.", function (_a) {
            var find = _a.find, record = _a.record;
            var element = find("html/event/hover-in").element;
            return [element.add("tag", "html/hovered")];
        })
            .commit("When an element is left, clear it's hovered.", function (_a) {
            var find = _a.find, record = _a.record;
            var element = find("html/event/hover-out").element;
            return [element.remove("tag", "html/hovered")];
        })
            .watch("When an element is hoverable, it subscribes to mouseover/mouseout", function (_a) {
            var find = _a.find, record = _a.record;
            var elemId = find("html/listener/hover");
            var instanceId = find("html/instance", { element: elemId });
            return [
                record({ listener: "hover", elemId: elemId, instanceId: instanceId })
            ];
        })
            .asObjects(function (_a) {
            var adds = _a.adds, removes = _a.removes;
            for (var _i = 0, _b = Object.keys(adds); _i < _b.length; _i++) {
                var e = _b[_i];
                var _c = adds[e], listener = _c.listener, elemId = _c.elemId, instanceId = _c.instanceId;
                var instance = _this.getInstance(instanceId);
                if (!instance.listeners)
                    instance.listeners = {};
                instance.listeners[listener] = true;
            }
            for (var _d = 0, _e = Object.keys(removes); _d < _e.length; _d++) {
                var e = _e[_d];
                var _f = removes[e], listener = _f.listener, elemId = _f.elemId, instanceId = _f.instanceId;
                var instance = _this.getInstance(instanceId);
                if (!instance || !instance.listeners)
                    continue;
                instance.listeners[listener] = false;
            }
        });
        this.program
            .commit("Remove hover-in events", function (_a) {
            var find = _a.find;
            var event = find("html/event/hover-in");
            return [event.remove()];
        })
            .commit("Remove hover-out events", function (_a) {
            var find = _a.find;
            var event = find("html/event/hover-out");
            return [event.remove()];
        });
    };
    return HTMLWatcher;
}(dom_1.DOMWatcher));
exports.HTMLWatcher = HTMLWatcher;
watcher_1.Watcher.register("html", HTMLWatcher);
//# sourceMappingURL=html.js.map